#!/usr/bin/env python

###############################################################################################
#
#  PC2 hack logger - Justin Carlson (justinca@cs.cmu.edu)
# 
#  This script monitors the sumtime.html file output by a pc2 scoreboard client
#  for changes.  It logs and timestamps changes in the scoring state to a custom
#  (but human readable) format which can be used to generate a merged html scoreboard
#  for a virtual cross-site competition.  Note the merging part is purely theoretical
#  at this point; it's not been written yet.
#
#  The script attempts to be reasonably robust to the reality that there is no file
#  locking; if it doesn't parse a valid xml tree from the scoreboard file, it assumes
#  that it hit a mid-update case and reattempts on the next polling interval.  
#
#  The script is not particularly robust against malicious team name creation; to keep
#  from messing it up, teams should not have any html tags in their names (this may mess
#  up the html output as it is, I haven't checked) and they also should be devoid of semicolons.
#  There are probably other subtle things in team names that could mess things up, but it's not
#  a big concern unless your site administrator is the malicious party, in which case you are 
#  already screwed.
#
#  -J 2008/08/24
#
#################################################################################################
import xml
from xml.etree import ElementTree
import os
import sys
import time
import re
import StringIO

import sumtime_reader

#############################
#
#  Configuration parameters
# 
#############################

# Interval, in seconds, at which we poll the html file for changes
# Note this is a floating point value; values <1 can be used.
_POLL_INTERVAL = 1

# Log copies of the sumtime.html file every time it changes
_DEBUG_LOGGING = True

############################
#
# The good stuff
# 
############################

class BoardState:
    # Create a new board state by parsing the file.  IOError will be thrown
    # if the file cannot be read or parsed.  If no file is given, a blank
    # state is generated.
    def __init__(self, file=None):
        self.teams = {}
        if not file:
            return
        self._parse(file)
        
    # Log the changes between this state and the newer state to the given file
    def log_diff(self, newer, outfile):
        for k in newer.teams.keys():
            if (not k in self.teams) or (self.teams[k] != newer.teams[k]):
                print "Updating team", k
                outfile.write(str(time.time()) + ";" + k)
                for score in newer.teams[k]:
                    outfile.write(";" + str(score[0]) + "/" + str(score[1]))
                outfile.write("\n");
        outfile.flush()

    
    def _parse(self, file):
        tmp = sumtime_reader.parse_board(file)
        if not tmp:
            raise IOError("Failed to parse board")
        self.teams = tmp
        
        
def main():

    debug_log_num = 0
    
    if len(sys.argv) != 3:
        print "Usage:", os.path.basename(sys.argv[0]), "<sumtime.html location> <output log>"
        sys.exit(-1)
        
    infile = sys.argv[1]
    outfile = open(sys.argv[2], "w")
    outfile.write("LOGPC2 " + str(time.time()) + "\n")

    prev_mod_time = 0;
    
    board_state = BoardState()

    while(True):
        time.sleep(_POLL_INTERVAL)
        try:
            st = os.stat(infile)
        except OSError:
            print "WARNING: Couldn't stat", infile + "...maybe it doesn't exist yet?"
            continue
        if (st.st_mtime > prev_mod_time):
            if _DEBUG_LOGGING:
                os.system("cp " + infile + " " + os.path.dirname(sys.argv[2]) + "/debug_log." + ("%05i" % debug_log_num))
                debug_log_num += 1

            try:
                new_board_state = BoardState(infile)
            except IOError:
                # Probably a synchronization error; we likely read it mid update
                print "IO Error reading", infile + ".  Unless this is happening a lot, it's probably OK."
                continue

                

            board_state.log_diff(new_board_state, outfile)
            board_state = new_board_state
            prev_mod_time = st.st_mtime
    
    
if __name__ == "__main__":
    main()
